#!/usr/bin/env python3

__author__ = "j3p0uk"

import numpy as np
import textwrap
from . import sheets


class NetUMLDraw(object):
    """
    @startuml
    frame "Room 1" as 1 {
        frame "Switch 1" as 2 {
            queue "Port1" as 3
            queue "Port2" as 4
        }
        frame "Device 1" as 5 {
            queue "LAN" as 6
        }
        frame "Device 2" as 7 {
            queue "LAN" as 8
        }
    }
    frame "Room 2" as 9 {
        frame "Switch 2" as 10 {
            queue "Port1" as 11
            queue "Port2" as 12
        }
        frame "Device 3" as 13 {
            queue "LAN" as 14
        }
    }
    3 -- 6
    4 -- 8
    11 -- 14
    @enduml
    """
    start = '@startuml'
    end = '@enduml'

    def __init__(self):
        self.data = None
        self.obj = 0
        self.rooms = {}
        self.indent = '    '
        self.sheets = sheets.Sheets()

    def read_data(self):
        self.obj = 0
        self.rooms = {}
        self.data = self.sheets.get_values()

    def print_data(self):
        # Skipping the header line
        for line in self.data[1:]:
            print("{}".format(line))

    def process_data(self):
        # Get the row index alongside the data
        for obj, line in enumerate(self.data[1:], 2):
            # line[1]: Rooms - these are frames that contain frames
            # line[2]: Device - these are frames that contain queues as ports
            # line[3]: Port - these are queues to represent ports on a device
            # line[5]: Connections - these connect queues
            self.rooms.setdefault(line[1], {})
            if len(line) < 6:
                self.rooms[line[1]].setdefault(line[2], {})[line[3]] = (obj, None)
            else:
                # Split off just the number from the A1 notation returned
                conn = None
                try:
                    conn = int(line[5].lstrip("=A"))
                except ValueError:
                    print("Could not read int from '{}'.lstrip(\"=A\")".format(line[5]))
                self.rooms[line[1]].setdefault(line[2], {})[line[3]] = (obj, conn)

    def room_string(self, room):
        self.obj += 1
        return 'frame "{}" as {} {{'.format(room, self.obj)

    def device_string(self, device):
        self.obj += 1
        return textwrap.indent('frame "{}" as {} {{'.format(device, self.obj), self.indent)

    def port_string(self, conn):
        self.obj += 1
        return textwrap.indent('queue "{}" as {}'.format(conn, self.obj), self.indent + self.indent)

    def write_plant_uml(self):
        print(self.start)
        lookup = {}
        for room in self.rooms.keys():
            print(self.room_string(room))
            for device in self.rooms[room].keys():
                print(self.device_string(device))
                for conn in self.rooms[room][device].keys():
                    print(self.port_string(conn))
                    lookup[self.obj] = {}
                    this, that = self.rooms[room][device][conn]
                    lookup[self.obj]['this'] = this
                    lookup[self.obj]['that'] = that
                print(textwrap.indent('}', self.indent))
            print('}')
        conns = []
        for k, v in lookup.items():
            for key in lookup.keys():
                if lookup[key]['this'] == v['that']:
                    conns.append('{} -- {}'.format(min(k, key), max(k, key)))
                    break
        print('\n'.join(np.unique(conns)))
        print(self.end)



#!/usr/bin/env python

__author__ = "j3p0uk"

import net_uml_draw
import nose
import sys
import unittest.mock


class ExitException(Exception):
    pass


class TestNetUMLDrawCli(object):

    @unittest.mock.patch('net_uml_draw.NetUMLDraw', autospec=True)
    def test_uml_draw_cli(self, mock_nud):
        sys.argv = ['net-uml-draw']
        with unittest.mock.patch('sys.exit', side_effect=ExitException) as mock_exit:
            nose.tools.assert_raises(ExitException, net_uml_draw.cli.main)
            mock_exit.assert_called_with(0)
            calls = [unittest.mock.call().read_data(),
                     unittest.mock.call().process_data(),
                     unittest.mock.call().write_plant_uml()]
            print(mock_nud.method_calls)
            nose.tools.ok_(mock_nud.method_calls == calls)



#!/usr/bin/env python

__author__ = "j3p0uk"

import io
import net_uml_draw
import nose.tools
import unittest

FAKE_VALUES = [["header"], ["data"]]


class TestNetUMLDrawSheets(object):
    def __init__(self):
        self.sheets = net_uml_draw.sheets.Sheets()

    def test_get_creds_flow(self):
        creds = None
        with unittest.mock.patch('os.path.exists', return_value=False) as mock_exists, \
                unittest.mock.patch('pickle.dump') as mock_pickle, \
                unittest.mock.patch('net_uml_draw.sheets.InstalledAppFlow',
                                    return_value=unittest.mock.MagicMock()) as mock_flow:
            creds = self.sheets.get_creds()
        nose.tools.ok_(mock_exists.call_count == 1)
        nose.tools.ok_(mock_pickle.call_count == 1)
        nose.tools.ok_(creds is not None)
        expected_creds_calls = [
            unittest.mock.call.from_client_secrets_file(
                'credentials.json', ['https://www.googleapis.com/auth/spreadsheets.readonly']),
            unittest.mock.call.from_client_secrets_file().run_local_server(port=0)]
        nose.tools.ok_(mock_flow.mock_calls == expected_creds_calls,
                       "Calls are {} not {}".format(mock_flow.mock_calls, expected_creds_calls))

    def test_get_creds_pickle(self):
        creds = None
        with unittest.mock.patch('os.path.exists', return_value=True) as mock_exists, \
                unittest.mock.patch('pickle.load',
                                    return_value=unittest.mock.MagicMock()) as mock_pickle, \
                unittest.mock.patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            creds = self.sheets.get_creds()
        nose.tools.ok_(mock_exists.call_count == 1)
        nose.tools.ok_(mock_pickle.call_count == 1)
        printed = mock_stdout.getvalue().strip()
        nose.tools.ok_(printed == 'Loaded token from: token.pickle',
                       "Output was {}".format(printed))
        nose.tools.ok_(creds is not None)
        expected_creds_calls = [unittest.mock.call.__bool__(),
                                unittest.mock.call.valid.__bool__()]
        nose.tools.ok_(creds.mock_calls == expected_creds_calls,
                       "Calls are {} not {}".format(creds.mock_calls, expected_creds_calls))

    def test_get_creds_pickle_refresh(self):
        creds = None
        creds_mock = unittest.mock.MagicMock()
        creds_mock.valid = False
        with unittest.mock.patch('os.path.exists', return_value=True) as mock_exists, \
                unittest.mock.patch('pickle.load', return_value=creds_mock) as mock_load, \
                unittest.mock.patch('pickle.dump') as mock_dump, \
                unittest.mock.patch('net_uml_draw.sheets.Request',
                                    return_value='request') as mock_request, \
                unittest.mock.patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            creds = self.sheets.get_creds()
        nose.tools.ok_(mock_exists.call_count == 1)
        nose.tools.ok_(mock_load.call_count == 1)
        nose.tools.ok_(mock_dump.call_count == 1)
        nose.tools.ok_(mock_request.call_count == 1)
        printed = mock_stdout.getvalue().strip()
        nose.tools.ok_(printed == 'Loaded token from: token.pickle',
                       "Output was {}".format(printed))
        nose.tools.ok_(creds is not None)
        expected_creds_calls = [unittest.mock.call.__bool__(),
                                unittest.mock.call.__bool__(),
                                unittest.mock.call.expired.__bool__(),
                                unittest.mock.call.refresh_token.__bool__(),
                                unittest.mock.call.refresh('request')]
        nose.tools.ok_(creds.mock_calls == expected_creds_calls,
                       "Calls are {} not {}".format(creds.mock_calls, expected_creds_calls))

    @unittest.mock.patch.dict('os.environ', {'DATA_RANGE': 'range'})
    def test_get_values_no_data_range(self):
        values = self.sheets.get_values()
        nose.tools.ok_(values is None)

    @unittest.mock.patch.dict('os.environ', {'SHEET': "sheet"})
    def test_get_values_no_sheet(self):
        values = self.sheets.get_values()
        nose.tools.ok_(values is None)

    def read_sheet_side_effect(self, sheet, data_range):
        self.sheets.values = FAKE_VALUES

    @unittest.mock.patch.dict('os.environ', {'SHEET': "sheet", 'DATA_RANGE': 'range'})
    def test_get_values_env(self):
        self.sheets.read_sheet = unittest.mock.MagicMock()
        self.sheets.read_sheet.side_effect = self.read_sheet_side_effect
        values = self.sheets.get_values()
        nose.tools.ok_(values == FAKE_VALUES, "Values is {} not {}".format(values, FAKE_VALUES))

    def test_get_values_arg(self):
        self.sheets.read_sheet = unittest.mock.MagicMock()
        self.sheets.read_sheet.side_effect = self.read_sheet_side_effect
        values = self.sheets.get_values("sheet", "range")
        print(self.sheets.read_sheet.calls)
        nose.tools.ok_(values == FAKE_VALUES, "Values is {} not {}".format(values, FAKE_VALUES))

    def test_read_sheet(self):
        self.sheets.get_creds = unittest.mock.MagicMock()
        net_uml_draw.sheets.build = unittest.mock.MagicMock()
        service = unittest.mock.MagicMock()

        net_uml_draw.sheets.build.return_value = service
        service.spreadsheets().values().get().execute().get.return_value = FAKE_VALUES

        with unittest.mock.patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            self.sheets.read_sheet("sheet", "range")
        printed = mock_stdout.getvalue().strip()
        nose.tools.ok_(printed == 'Found 2 rows of data', "Output was {}".format(printed))
        nose.tools.ok_(self.sheets.values == FAKE_VALUES,
                       "Values is {} not {}".format(self.sheets.values, FAKE_VALUES))

    def test_read_sheet_fail(self):
        self.sheets.get_creds = unittest.mock.MagicMock()
        net_uml_draw.sheets.build = unittest.mock.MagicMock()
        service = unittest.mock.MagicMock()

        net_uml_draw.sheets.build.return_value = service
        service.spreadsheets().values().get().execute().get.return_value = None

        with unittest.mock.patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            self.sheets.read_sheet("sheet", "range")
        printed = mock_stdout.getvalue().strip()
        nose.tools.ok_(printed == 'No data found.', "Output was {}".format(printed))
        nose.tools.ok_(self.sheets.values is None,
                       "Values is {} not None".format(self.sheets.values))



#!/usr/bin/env python

__author__ = "j3p0uk"

import io
import net_uml_draw
import nose.tools
import textwrap
import unittest

DATA = [
    ["Display Name and Port", "Room", "Device", "Port", "MAC", "Connected to", "Display Name"],
    ["", "Room 1", "Dev 1", "WiFi", "", "=A2", "Device 1"],
    ["", "Room 1", "Dev 1", "1", ""],
    ["", "Room 1", "Dev 1", "2", "", "=A5", ""],
    ["", "Room 2", "Dev 2", "LAN", "", "=A4", ""],
    ["", "Room 3", "Dev 3", "WiFi", "", "=A2", ""],
    ["", "Room 3", "Dev 4", "WiFi", "", "", "Display Name"]]


class TestNetUMLDraw(object):

    def __init__(self):
        self.nud = net_uml_draw.NetUMLDraw()
        self.nud.sheets = unittest.mock.MagicMock()

    def test_net_uml_draw(self):
        nose.tools.ok_(self.nud.data is None)
        nose.tools.ok_(self.nud.obj == 0)
        nose.tools.ok_(self.nud.rooms == {})
        nose.tools.ok_(self.nud.indent == '    ')

    def test_read_data(self):
        self.nud.data = [["header"], ["data"]]
        self.nud.obj = 10
        self.nud.rooms = {"room1": {}, "room2": {}}
        self.nud.sheets.get_values.return_value = [["header"], ["new_data"]]
        self.nud.read_data()
        nose.tools.ok_(self.nud.data == [["header"], ["new_data"]])
        nose.tools.ok_(self.nud.obj == 0)
        nose.tools.ok_(self.nud.rooms == {})

    def test_print_data(self):
        data = ["data"]
        self.nud.data = [["header"], data]
        with unittest.mock.patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            self.nud.print_data()
        printed_data = mock_stdout.getvalue().strip()
        print(printed_data)
        nose.tools.ok_(printed_data == data.__str__(), "Output {} doesn't match {}".format(
            printed_data, data))

    def test_process_data(self):
        self.nud.data = DATA
        self.nud.process_data()
        expected_dict = {'Room 1': {'Dev 1': {'WiFi': (2, 2), '1': (3, None), '2': (4, 5)}},
                         'Room 2': {'Dev 2': {'LAN': (5, 4)}},
                         'Room 3': {'Dev 3': {'WiFi': (6, 2)}, 'Dev 4': {'WiFi': (7, None)}}}
        nose.tools.ok_(self.nud.rooms == expected_dict, "Rooms '{}' doesn't match '{}'".format(
            self.nud.rooms, expected_dict))

    def test_room_string(self):
        self.nud.obj = 0
        ret_string = self.nud.room_string("Room 1")
        expected_string = 'frame "Room 1" as 1 {'
        nose.tools.ok_(ret_string == expected_string, "Return '{}' doesn't match '{}'".format(
            ret_string, expected_string))

    def test_device_string(self):
        self.nud.obj = 1
        ret_string = self.nud.device_string("Device 1")
        expected_string = textwrap.indent('frame "Device 1" as 2 {', self.nud.indent)
        nose.tools.ok_(ret_string == expected_string, "Return '{}' doesn't match '{}'".format(
            ret_string, expected_string))

    def test_port_string(self):
        self.nud.obj = 2
        ret_string = self.nud.port_string("Port 1")
        expected_string = textwrap.indent('queue "Port 1" as 3', self.nud.indent + self.nud.indent)
        nose.tools.ok_(ret_string == expected_string, "Return '{}' doesn't match '{}'".format(
            ret_string, expected_string))

    def test_write_plant_uml(self):
        self.nud.data = DATA
        self.nud.process_data()
        with unittest.mock.patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            self.nud.write_plant_uml()
        ret_string = mock_stdout.getvalue().strip()
        expected_string = '''
@startuml
frame "Room 1" as 1 {
    frame "Dev 1" as 2 {
        queue "WiFi" as 3
        queue "1" as 4
        queue "2" as 5
    }
}
frame "Room 2" as 6 {
    frame "Dev 2" as 7 {
        queue "LAN" as 8
    }
}
frame "Room 3" as 9 {
    frame "Dev 3" as 10 {
        queue "WiFi" as 11
    }
    frame "Dev 4" as 12 {
        queue "WiFi" as 13
    }
}
3 -- 11
3 -- 3
5 -- 8
@enduml'''.strip()
        nose.tools.ok_(ret_string == expected_string, "Return '{}' doesn't match '{}'".format(
            ret_string, expected_string))



__author__ = 'j3p0uk'

import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
version_file = os.path.join(here, 'VERSION')

# Get the long description from the README file
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

if not os.path.isfile(version_file):
    BUILD_NUMBER = os.environ.get('TRAVIS_BUILD_NUMBER', '0')
    with open(version_file, encoding='utf-8', mode='w') as f:
        f.write(BUILD_NUMBER)
else:
    with open(version_file, encoding='utf-8') as f:
        BUILD_NUMBER = f.read().strip()

setup(
    name='net_uml_draw',
    version='0.' + BUILD_NUMBER,

    description='Write PlantUML from a Google Sheet network description',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/j3p0uk/net-uml-draw',

    author='JP Sullivan (j3p0uk)',
    author_email='j3p0uk@gmail.com',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
    ],

    keywords='network_diagram plantuml google_sheets',

    install_requires=[
        "google-api-python-client",
        "google-auth-httplib2",
        "google-auth-oauthlib",
        "numpy"
    ],

    platforms=['Any'],

    scripts=[],

    provides=['net_uml_draw'],

    packages=find_packages(exclude=['docs', 'test']),
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'net-uml-draw=net_uml_draw.cli:main',
        ],
    },

    data_files=[("", ["LICENSE", "VERSION"])],

    project_urls={
        'Bug Reports': 'https://github.com/j3p0uk/net-uml-draw/issues',
        'Source': 'https://github.com/j3p0uk/net-uml-draw/',
    },

    zip_safe=False,
)



#!/usr/bin/env python3

__author__ = "j3p0uk"

import pickle
import os
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request


class Sheets(object):
    """
    Encapsulate interactions with Google Sheets
    """
    def __init__(self):
        self.scopes = ['https://www.googleapis.com/auth/spreadsheets.readonly']
        self.values = None

    # Override magic functions to make this object indexable into the sheet values

    def get_creds(self):
        creds = None
        # The file token.pickle stores the user's access and refresh tokens, and is
        # created automatically when the authorization flow completes for the first
        # time.
        pickle_location = 'token.pickle'
        if os.path.exists(pickle_location):
            with open(pickle_location, 'rb') as token:
                creds = pickle.load(token)
                print("Loaded token from: {}".format(pickle_location))
        # If there are no (valid) credentials available, let the user log in.
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    'credentials.json', self.scopes)
                creds = flow.run_local_server(port=0)
            # Save the credentials for the next run
            with open(pickle_location, 'wb') as token:
                pickle.dump(creds, token)
        return creds

    def read_sheet(self, sheet_id, data_range):
        self.service = build('sheets', 'v4', credentials=self.get_creds())

        # Call the Sheets API
        self.sheet = self.service.spreadsheets()
        self.result = self.sheet.values().get(spreadsheetId=sheet_id, range=data_range,
                                              valueRenderOption='FORMULA').execute()
        self.values = self.result.get('values', [])

        if not self.values:
            print('No data found.')
        else:
            print('Found {} rows of data'.format(len(self.values)))

    def get_values(self, sheet=None, data_range=None):
        if sheet is None:
            sheet = os.environ.get('SHEET')
        if data_range is None:
            data_range = os.environ.get('DATA_RANGE')

        if sheet is None or data_range is None:
            print("Need both SHEET {} and DATA_RANGE {} to get values".format(sheet, data_range))
            self.values = None
        else:
            if self.values is None:
                self.read_sheet(sheet, data_range)
        return self.values



#!/usr/bin/env python3

__author__ = "j3p0uk"

import net_uml_draw
import sys


def main():
    status = 0
    # Check arguments
    #   * Sheet Name
    #   * Key Column IDs
    #   * Output file

    # Create diagram
    nud = net_uml_draw.NetUMLDraw()
    nud.read_data()
    nud.process_data()
    nud.write_plant_uml()

    # Exit with status
    sys.exit(status)
